import { Injectable } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import { Observable } from "rxjs";
import { CapabilityRequest } from "../model/capability-request";
import { CapabilityVersion } from "../model/capability-version";
import { JsonObject } from "@angular/compiler-cli/ngcc/src/packages/entry_point";
import { ActivatedRoute, Router } from "@angular/router";

@Injectable({
  providedIn: "root",
})
export class CapabilityRequestService {
  baseUrl = "/workspaces";
  constructor(
    private httpClient: HttpClient,
    private router: Router,
    private route: ActivatedRoute,
  ) {}

  /**
   * Returns URL for GETting capability request definition
   * @param id  Capability request ID
   * @return string  The "GET capability request" HTTP url
   */
  public getCapabilityRequestURL(id: number): string {
    return `capability/request/${id}`;
  }

  /**
   * Gets JSON representation of a capability request given an ID
   * @param id  The ID of the request
   * @return Observable<CapabilityRequest>  Result of REST caall to get the request
   */
  public getCapabilityRequest(id: number): Observable<CapabilityRequest> {
    return this.httpClient.get<CapabilityRequest>(this.getCapabilityRequestURL(id));
  }

  /**
   * Gets JSON representation of a list of active capability requests for a given capability
   * @param capabilityName  The name of the capability
   */
  public getActiveCapabilityRequests(capabilityName: string): Observable<JsonObject> {
    const url = `capability/${capabilityName}/request`;
    return this.httpClient.get<JsonObject>(url);
  }

  /**
   * Gets URL to retrieve a list of active capability requests for a given capability
   * @param capabilityName  The name of the capability
   */
  public getActiveCapabilityRequestsUrl(capabilityName: string): string {
    return `capability/${capabilityName}/request`;
  }

  /**
   * Gets JSON representation of a list of active capability requests for a given capability
   * @param capabilityName  The name of the capability
   */
  public getCreatedCapabilityRequests(capabilityName: string): Observable<JsonObject> {
    const url = `capability/${capabilityName}/created-requests`;
    return this.httpClient.get<JsonObject>(url);
  }

  /**
   * Gets URL to retrieve a list of active capability requests for a given capability
   * @param capabilityName  The name of the capability
   */
  public getCreatedCapabilityRequestsUrl(capabilityName: string): string {
    return `capability/${capabilityName}/created-requests`;
    
  }

  /**
   * Returns URL for GETting the latest version of a capability request
   * @param id  Capability request ID
   * @return string  The "GET latest version" HTTP url
   */
  public getLatestVersionURL(id: number): string {
    return `capability/request/${id}/version/latest`;
  }

  /**
   * Get latest version for capability request with given ID
   * @param id  ID of request to get version for
   * @return Observable<CapabilityVersion>  Result of REST call to get the version
   */
  public getLatestVersion(id: number): Observable<CapabilityVersion> {
    return this.httpClient.get<CapabilityVersion>(this.getLatestVersionURL(id));
  }

  /**
   * Add file to capability request version
   * @param requestId  ID of request who's latest version will receive the file
   * @param filename  Name of file
   * @param content  Content of file
   * @return  Observable<string>  Result of REST call to add the file
   * @private
   */
  addVersionFile(requestId: number, filename: string, content: string): Observable<string> {
    const addFileUrl = `capability/request/${requestId}/version/file/add`;
    return this.httpClient.post<string>(
      addFileUrl,
      JSON.stringify({ parameters: { filename: filename, content: content } }),
    );
  }

  /**
   * Create a new version of the specified capability request with specified settings
   * @param request_id  ID of request to add new version to
   * @param parametersString  Parameters of the new version as a string
   * @param inputFiles  Input files to give to the new version
   */
  public createNewVersion(
    request_id: number,
    parametersString: string,
    inputFiles: FileList,
  ): Observable<CapabilityVersion> {
    const parameters = JSON.parse(parametersString);
    const newVersionUrl = `capability/request/${request_id}/version/create`;

    return this.httpClient.post<CapabilityVersion>(
      newVersionUrl,
      JSON.stringify({ parameters: parameters }),
    );
  }

  /**
   * Add all files from a file list to the latest version of a given request
   * @param requestId  ID for request to add files to
   * @param inputFileList  List of files to be added
   * @return Array<string>  List of filenames that were successfully added
   */
  public addFilesToVersion(requestId: string, inputFileList: FileList) {
    const addFileObservable = {
      next: (response) => {
        console.log("Successfully added file to version.");
      },
      error: (error) => console.error("Error when adding file to version:" + error),
    };

    for (let i = 0; i < inputFileList.length; i++) {
      const currentFile = inputFileList.item(i);
      const filename = currentFile.name;
      currentFile
        .text()
        .then((content) =>
          this.addVersionFile(parseInt(requestId), filename, content).subscribe(addFileObservable),
        );
    }
  }

  /**
   * Redirect to status page of capability request with given ID
   * @param requestId  Given request ID
   */
  public redirectToRequestStatusPage(requestId: string) {
    const statusPageUrl = this.router.serializeUrl(
      this.router.createUrlTree([`${this.baseUrl}/request-status/${requestId}`], {
        relativeTo: this.route,
      }),
    );
    // this.router.navigate([`request-status/${requestId}`], { relativeTo: this.route });
    // Open URL in new tab
    window.open(statusPageUrl, "_blank");
  }
}
